/********************************************************************
*--------------------------------------------------------------------
*
*    Titolo        :  C Enhanced - es6C_2
*                     Contatore up/down su display a 7 segmenti.
*                   - Un pulsante Up fa avanzare il contatore fino
*                     alla cifra F, dopo di che si accende il punto
*                     decimale e il pulsante non ha pi effetto.
*                     Occorrer sbloccare la situazione con il
*                     il pulsante Down.
*                   - Un pulsante Down fa arretrare il conteggio fino
*                     alla cifra 0, dopo di che si accende il punto
*                     decimale e il pulsante non ha pi effetto.
*                     Occorrer sbloccare la situazione con il
*                     il pulsante Up.
*                     Reset  attivo e cancella il display.
*    Data          :  01-05-2011
*    Modificato il :
*    Versione      :  V0.0
*    Ref. Hardware :  PIC16F1619
*    Autore        :  afg
*--------------------------------------------------------------------
*********************************************************************
*  Impiego pin :
*  ----------------
*       16F1619 @ 20 pin
*
*                  |\/|
*             Vdd -|1   20|- Vss
*       RA5/CLKIN -|2   19|- RA0/ICSPDAT
*      RA4/CLKOUT -|3   18|- RA1/ICSPCLK
*        RA3/MCLR -|4   17|- RA2
*             RC5 -|5   16|- RCO
*             RC4 -|6   15|- RC1
*             RC3 -|7   14|- RC2
*             RC6 -|8   13|- RB4
*             RC7 -|9   12|- RB5
*             RB7 -|10  11|- RB6
*                  |______|
*
* Impiego pin:
*    Vdd      1: ++
*    RA5      2: In
*    RA4      3: In  PB2
*    RA3      4: MCLR
*    RC5      5: Out segm. e
*    RC4      6: Out segm. d
*    RC3      7: Out segm. f
*    RC6      8: Out segm. g
*    RC7      9: Out segm. dp
*    RB7     10: In
*    RB6     11: In
*    RB5     12: In
*    RB4     13: In
*    RC2     14: Out segm. c
*    RC1     15: Out segm. b
*    RC0     16: Out segm. a
*    RA2     17: In  PB1
*    RA1     18: In
*    RA0     19: In
*    Vss              20: --
*
* Funzioni:
*    RA0/AN0/DAC1OUT1/C1IN+
*    RA1/AN1/Vref+/C1IN0-C2IN0-
*    RA2/AN2/[T0CKI/CWG1IN]/ZCD1IN/INT
*    RA3/MCLR/[T6IN/SMTWIN2]
*    RA4/AN4/[T1G/SMTSIG1]
*    RA5/[T1CK/T2IN/SMTWIN1]/CLCIN3
*    RB4/AN10/[SDI]
*    RB5/AN11/[RX]
*    RB6/[SCK]
*    RB7/[CK]
*    RC0/AN4/C2IN+/[T5CKI]
*    RC1/AN5/C1IN1-/C2IN1-
*    RC2/AN6/C1IN2-/C2IN2-
*    RC3/AN7/C1IN3-/C2IN3-/[T5G/CCP2/CLCIN0/ATCC]
*    RC4/[T3G/CLCIN1]/HIC4
*    RC5/[T3CKI/CCP1/ATIN]/HIC5
*    RC6/AN8/[!SS]
*    RC7/AN9
*
* [] rilocabili
* Tutti i pin con wpu e ioc
*
* Note: - INTOSC default @ 500kHz
*
********************************************************************/

#include "C:\Corso_C\MyIncludes\conf1619_0.h"

#include <xc.h>

#define _XTAL_FREQ  500000
#define Up    PORTAbits,RA2
#define Down  PORTAbits.RA4
#define dp    LATCbits.LATC7

// 7 segments lookup table per cifre da 0 a F
// Rx0=segm.a, Rx1= segm.b, ...Rx7= segm.dp
  unsigned char tab7segm[16] = {
		  0x3F,  // 0
	  0x06,  // 1
	  0x5B,  // 2
	  0x4F,  // 3
	  0x66,  // 4
	  0x6D,  // 5
	  0x7D,  // 6
	  0x07,  // 7
	  0x7F,  // 8
	  0x6F,  // 9
	  0x77,  // A
	  0x7C,  // b
	  0x39,  // C
	  0x5E,  // d
	  0x79,  // E
	  0x71   // F
  };

/************************* MAIN PROGRAM ****************************/
void main(void){

 unsigned char digit=0;          // cifra da inviare al display

    // Inizializza I/O
    ANSELA = 0;                  // disabilita analogiche su
    ANSELB = 0;                  // tutti port
    ANSELC = 0;

    // protezione pin non usati
    OPTION_REGbits,nWPUEN = 0;   // abilitazione generale wpu

    // inizio loop con display spento
    LATC = 0;                    // preset latch C a 0 (segmenti spenti)
    TRISC = 0;                   // tutti i pin di PORTC come uscite

    LATC = tab7segm[digit];      // da lut a display = 0

  while(1){                      // loop principale
     if(Up==0){                  // se Up premuto, count up
      if(digit<15){              // se cifre non oltre F
       digit++;                  // incrementa digit
       LATC = tab7segm[digit];   // da lut a display
       dp=0;                     // no dp
       __delay_ms(30);           // debounce
      }else{                     // se F
	   dp=1;                     // accendi dp
	   }
     while(Up==0);               // attendi per Up rilasciato
        __delay_ms(30);          // debounce
     }

     if(Down==0){                // se Down premuto, count down
      if(digit>0){               // se cifre>0
       digit--;                  // decrementa digit
       LATC = tab7segm[digit];   // da lut a display
       dp=0;                     // no dp
       __delay_ms(30);           // debounce
      }else{                     // se = 0
	   dp=1;                     // accendi dp
	   }
     while(Down==0);             // attendi per Down rilasciato
        __delay_ms(30);          // debounce
     }
  }
}
